package javax.swing;

import java.util.EventListener;
import javax.swing.event.ChangeListener;
import javax.swing.event.EventListenerList;
import javax.swing.event.ChangeEvent;
import java.io.Serializable;

public class DefaultBoundedRangeModel implements BoundedRangeModel, Serializable
{
    protected transient ChangeEvent changeEvent;
    protected EventListenerList listenerList;
    private int value;
    private int extent;
    private int min;
    private int max;
    private boolean isAdjusting;
    
    public DefaultBoundedRangeModel() {
        this.changeEvent = null;
        this.listenerList = new EventListenerList();
        this.value = 0;
        this.extent = 0;
        this.min = 0;
        this.max = 100;
        this.isAdjusting = false;
    }
    
    public DefaultBoundedRangeModel(int value, int extent, int min, int max) {
        this.changeEvent = null;
        this.listenerList = new EventListenerList();
        this.value = 0;
        this.extent = 0;
        this.min = 0;
        this.max = 100;
        this.isAdjusting = false;
        if (max >= min && value >= min && value + extent >= value && value + extent <= max) {
            this.value = value;
            this.extent = extent;
            this.min = min;
            this.max = max;
            return;
        }
        throw new IllegalArgumentException("invalid range properties");
    }
    
    @Override
    public int getValue() {
        return this.value;
    }
    
    @Override
    public int getExtent() {
        return this.extent;
    }
    
    @Override
    public int getMinimum() {
        return this.min;
    }
    
    @Override
    public int getMaximum() {
        return this.max;
    }
    
    @Override
    public void setValue(int min) {
        min = Math.min(min, Integer.MAX_VALUE - this.extent);
        int max = Math.max(min, this.min);
        if (max + this.extent > this.max) {
            max = this.max - this.extent;
        }
        this.setRangeProperties(max, this.extent, this.min, this.max, this.isAdjusting);
    }
    
    @Override
    public void setExtent(int b) {
        int max = Math.max(0, b);
        if (this.value + max > this.max) {
            max = this.max - this.value;
        }
        this.setRangeProperties(this.value, max, this.min, this.max, this.isAdjusting);
    }
    
    @Override
    public void setMinimum(int n) {
        int max = Math.max(n, this.max);
        int max2 = Math.max(n, this.value);
        this.setRangeProperties(max2, Math.min(max - max2, this.extent), n, max, this.isAdjusting);
    }
    
    @Override
    public void setMaximum(int a) {
        int min = Math.min(a, this.min);
        int min2 = Math.min(a - min, this.extent);
        this.setRangeProperties(Math.min(a - min2, this.value), min2, min, a, this.isAdjusting);
    }
    
    @Override
    public void setValueIsAdjusting(boolean b) {
        this.setRangeProperties(this.value, this.extent, this.min, this.max, b);
    }
    
    @Override
    public boolean getValueIsAdjusting() {
        return this.isAdjusting;
    }
    
    @Override
    public void setRangeProperties(int value, int extent, int min, int max, boolean isAdjusting) {
        if (min > max) {
            min = max;
        }
        if (value > max) {
            max = value;
        }
        if (value < min) {
            min = value;
        }
        if ((long)extent + (long)value > (long)max) {
            extent = max - value;
        }
        if (extent < 0) {
            extent = 0;
        }
        if (value != this.value || extent != this.extent || min != this.min || max != this.max || isAdjusting != this.isAdjusting) {
            this.value = value;
            this.extent = extent;
            this.min = min;
            this.max = max;
            this.isAdjusting = isAdjusting;
            this.fireStateChanged();
        }
    }
    
    @Override
    public void addChangeListener(ChangeListener l) {
        this.listenerList.add(ChangeListener.class, l);
    }
    
    @Override
    public void removeChangeListener(ChangeListener l) {
        this.listenerList.remove(ChangeListener.class, l);
    }
    
    public ChangeListener[] getChangeListeners() {
        return (ChangeListener[])this.listenerList.getListeners(ChangeListener.class);
    }
    
    protected void fireStateChanged() {
        Object[] listenerList = this.listenerList.getListenerList();
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == ChangeListener.class) {
                if (this.changeEvent == null) {
                    this.changeEvent = new ChangeEvent(this);
                }
                ((ChangeListener)listenerList[i + 1]).stateChanged(this.changeEvent);
            }
        }
    }
    
    @Override
    public String toString() {
        return this.getClass().getName() + "[" + ("value=" + this.getValue() + ", extent=" + this.getExtent() + ", min=" + this.getMinimum() + ", max=" + this.getMaximum() + ", adj=" + this.getValueIsAdjusting()) + "]";
    }
    
    public <T extends EventListener> T[] getListeners(Class<T> t) {
        return this.listenerList.getListeners(t);
    }
}
